// ENGR-2350 GPIO Test Project
// This project will test a rslk/launchpad board's available IO connections.

#include "engr2350_msp432.h"

#define OMIT_ENCODERS
// Add function prototypes here, as needed.
uint8_t pin_results[10][8][3];

void testMotors();

// Main Function
int main(void)
{
        uint8_t port,pin,result,failure;
    SysInit();

    printf("\r\n\n\n***************************************\r\n");
            printf("*     ENGR2350 GPIO Test Program      *\r\n");
            printf("***************************************\r\n\n");

    while(1){
        printf("Disconnect any circuit connected to the RSLK or devboard.\r\n");
        printf("Press and key to start GPIO test, press 'm' to test motors.\r\n");
        uint8_t cmd  = getchar();
        putchar('\n');
        if(cmd == 'm'){
            testMotors();
            continue;
        }
        failure = 0;
        // Loop through all ports (except J)
        for(port=1;port<11;port++){
            uint8_t max = 8;
            if(port == 10){
                max = 6;
            }
            for(pin=0;pin<(max);pin++){
                // Loop through pins on port
                // printf("%u.%u\t\t",port,pin);
#ifdef OMIT_ENCODERS
                if((port==1 && (pin==2 || pin==3))
                    || (port==5 && (pin==0 || pin==2))
                    || (port==10 && (pin==4 || pin==5))){
#else
                if((port==1 && (pin==2 || pin==3))
                    || (port==5 && (pin==0 || pin==2))){
#endif


                    pin_results[port-1][pin][0] = 0;
                    pin_results[port-1][pin][1] = 1;
                    pin_results[port-1][pin][2] = 0;
                    continue;
                }
                GPIO_setAsOutputPin(port,1<<pin);
                GPIO_setOutputLowOnPin(port,1<<pin);
                __delay_cycles(1e5);
                pin_results[port-1][pin][0] =  GPIO_getInputPinValue(port,1<<pin);
                GPIO_setOutputHighOnPin(port,1<<pin);
                __delay_cycles(1e5);
                pin_results[port-1][pin][1] =  GPIO_getInputPinValue(port,1<<pin);
                GPIO_setAsInputPin(port,1<<pin);
                pin_results[port-1][pin][2] = (pin_results[port-1][pin][0]!=0) || (pin_results[port-1][pin][1]!=1);
                failure += pin_results[port-1][pin][2];
                /*
                GPIO_setAsOutputPin(port,1<<pin);
                GPIO_setOutputLowOnPin(port,1<<pin);
                __delay_cycles(1e5);
                result = GPIO_getInputPinValue(port,1<<pin)==0;
                printf("%u\t",!result);
                if(!((port==5 && (pin==0 || pin==2)) || (port==10 && (pin==4 || pin==5)))){
                    failure += !result;
                }
                GPIO_setOutputHighOnPin(port,1<<pin);
                __delay_cycles(1e5);
                result = GPIO_getInputPinValue(port,1<<pin)==1;
                printf("%u\t\t",result);
                if(!((port==5 && (pin==0 || pin==2)) || (port==10 && (pin==4 || pin==5)))){
                    failure += !result;
                    printf("\r\n");
                }else{
                    printf("Encoder Pin\r\n");
                }
                GPIO_setAsInputPin(port,1<<pin);
                */

            }
        }
        if(failure){
            printf("\nTest failed for %u pin(s):\r\n",failure);
            printf("Port.Pin\tLow Test\tHigh Test\r\n");
            for(port=1;port<11;port++){
                uint8_t max = 8;
                if(port == 10){
                    max = 5;
                }
                for(pin=0;pin<max;pin++){
                    if(pin_results[port-1][pin][2]){
                        printf("%u.%u\t\t%u\t\t%u\r\n",port,pin,pin_results[port-1][pin][0],pin_results[port-1][pin][1]);
                    }
                }
            }
            putchar('\n');
        }else{
            printf("All pins tested good!\r\n\n");
        }

    }
}

// Add function declarations here as needed
void testMotors(){
    // Initialize the output pins:
    P2DIR |= BIT6 | BIT7;
    P2SEL0 |= BIT6 | BIT7;
    P2SEL1 &= ~(BIT6 | BIT7);
    // Timer configuration
    TA0CTL = TASSEL__SMCLK | ID__8;
    TA0CCTL3 = TA0CCTL4 = OUTMOD_3;
    TA0CCR0 = ((((uint32_t)24e6)>>3)*50)/1000000-1;    // Set period to 50 us (20 kHz) (-1 as it counts past TA0CCR0)
    TA0CCR3 = TA0CCR4 = TA0CCR0*.80;  // Slow
    TA0CTL |= MC__UP;
    GPIO_setAsOutputPin(GPIO_PORT_P5,GPIO_PIN4 | GPIO_PIN5);
    GPIO_setAsOutputPin(GPIO_PORT_P3,GPIO_PIN6 | GPIO_PIN7);

    printf("Motors running Forward.");
    GPIO_setOutputLowOnPin(GPIO_PORT_P5,GPIO_PIN4 | GPIO_PIN5);
    GPIO_setOutputHighOnPin(GPIO_PORT_P3,GPIO_PIN6 | GPIO_PIN7);
    __delay_cycles(24000000);
    printf("\r\nMotors running Reverse.");
    GPIO_setOutputHighOnPin(GPIO_PORT_P5,GPIO_PIN4 | GPIO_PIN5);
    __delay_cycles(24000000);
    printf("\r\nMotors OFF.\r\n\n");

    TA0CTL |= ~TIMER_A_CTL_MC_MASK;

    GPIO_setOutputLowOnPin(GPIO_PORT_P3,GPIO_PIN6 | GPIO_PIN7);
    GPIO_setOutputLowOnPin(GPIO_PORT_P5,GPIO_PIN4 | GPIO_PIN5);
    GPIO_setOutputLowOnPin(GPIO_PORT_P5,GPIO_PIN4 | GPIO_PIN5);
    GPIO_setAsInputPin(GPIO_PORT_P5,GPIO_PIN4 | GPIO_PIN5);
    GPIO_setAsInputPin(GPIO_PORT_P3,GPIO_PIN6 | GPIO_PIN7);
    GPIO_setAsInputPin(GPIO_PORT_P2,GPIO_PIN6 | GPIO_PIN7);


}

// Add interrupt functions last so they are easy to find
